@@ -1,6 +1,5 @@
 # et:ts=4
 # package.tcl
-# $Id: package.tcl 103400 2013-02-24 20:49:40Z cal@macports.org $
 #
 # Copyright (c) 2004, 2007-2008, 2010 The MacPorts Project
 # Copyright (c) 2002-2004 Apple Inc.
@@ -33,12 +32,9 @@
 # standard package load
 package provide mp_package 1.0
 
-package require portrpm 1.0
-package require portsrpm 1.0
 package require portpkg 1.0
 package require portmpkg 1.0
 package require portdmg 1.0
 package require portmdmg 1.0
-package require portdpkg 1.0
 package require portarchivefetch 1.0
 package require portunarchive 1.0
