@@ -1,6 +1,5 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portdestroot.tcl
-# $Id: portdestroot.tcl 109245 2013-08-11 11:51:15Z cal@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
 # Copyright (c) 2004 - 2005 Robert Shaw <rshaw@opendarwin.org>
@@ -34,6 +33,7 @@
 
 package provide portdestroot 1.0
 package require portutil 1.0
+package require fileutil
 
 set org.macports.destroot [target_new org.macports.destroot portdestroot::destroot_main]
 target_provides ${org.macports.destroot} destroot
@@ -48,13 +48,7 @@ namespace eval portdestroot {
 
 # define options
 options destroot.target destroot.destdir destroot.clean destroot.keepdirs destroot.umask \
-        destroot.violate_mtree destroot.asroot destroot.delete_la_files \
-        startupitem.autostart startupitem.create startupitem.executable \
-        startupitem.init startupitem.install startupitem.location \
-        startupitem.logevents startupitem.logfile startupitem.name \
-        startupitem.netchange startupitem.pidfile startupitem.plist \
-        startupitem.requires startupitem.restart startupitem.start \
-        startupitem.stop startupitem.type startupitem.uniquename
+        destroot.violate_mtree destroot.asroot destroot.delete_la_files
 commands destroot
 
 # Set defaults
@@ -68,42 +62,26 @@ default destroot.destdir {DESTDIR=${destroot}}
 default destroot.nice {${buildnicevalue}}
 default destroot.umask {$system_options(destroot_umask)}
 default destroot.clean no
-default destroot.keepdirs ""
+default destroot.keepdirs {}
 default destroot.violate_mtree no
 default destroot.delete_la_files {${delete_la_files}}
 
-default startupitem.name        {${subport}}
-default startupitem.uniquename  {org.macports.${startupitem.name}}
-default startupitem.plist       {${startupitem.uniquename}.plist}
-default startupitem.location    LaunchDaemons
-default startupitem.init        ""
-default startupitem.start       ""
-default startupitem.stop        ""
-default startupitem.restart     ""
-default startupitem.requires    ""
-default startupitem.executable  ""
-default startupitem.type        {$system_options(startupitem_type)}
-default startupitem.pidfile     ""
-default startupitem.logfile     ""
-default startupitem.logevents   no
-default startupitem.netchange   no
-default startupitem.install     {$system_options(startupitem_install)}
-
 set_ui_prefix
 
 proc portdestroot::destroot_getargs {args} {
-    if {(([option build.type] == "default" && [option os.platform] != "freebsd") || \
-         ([option build.type] == "gnu")) \
-        && [regexp "^(/\\S+/|)(g|gnu|)make(\\s+.*|)$" [option destroot.cmd]]} {
+    global build.type os.platform destroot.cmd destroot.target
+    if {((${build.type} eq "default" && ${os.platform} ne "freebsd") ||
+         (${build.type} eq "gnu"))
+        && [regexp "^(/\\S+/|)(g|gnu|)make(\\s+.*|)$" ${destroot.cmd}]} {
         # Print "Entering directory" lines for better log debugging
-        return "-w [option destroot.target]"
+        return "-w ${destroot.target}"
     }
 
-    return "[option destroot.target]"
+    return ${destroot.target}
 }
 
 proc portdestroot::destroot_start {args} {
-    global UI_PREFIX prefix subport porturl destroot os.platform destroot.clean portsharepath \
+    global UI_PREFIX prefix subport destroot os.platform destroot.clean portsharepath \
            destroot.umask destroot.asroot euid egid \
            applications_dir frameworks_dir
     variable oldmask
@@ -130,39 +108,35 @@ proc portdestroot::destroot_start {args} {
     set oldmask [umask ${destroot.umask}]
     set mtree [findBinary mtree ${portutil::autoconf::mtree_path}]
 
-    if { ${destroot.clean} == "yes" } {
+    if { ${destroot.clean} eq "yes" } {
         delete "${destroot}"
     }
 
     file mkdir "${destroot}"
-    if { ${os.platform} == "darwin" } {
-        system "cd \"${destroot}\" && ${mtree} -e -U -f [file join ${portsharepath} install macosx.mtree]"
+    if { ${os.platform} eq "darwin" } {
+        system -W ${destroot} "${mtree} -e -U -f [shellescape [file join ${portsharepath} install macosx.mtree]]"
         file mkdir "${destroot}${applications_dir}"
         file mkdir "${destroot}${frameworks_dir}"
     }
     file mkdir "${destroot}${prefix}"
-    system "cd \"${destroot}${prefix}\" && ${mtree} -e -U -f [file join ${portsharepath} install prefix.mtree]"
+    system -W ${destroot}${prefix} "${mtree} -e -U -f [shellescape [file join ${portsharepath} install prefix.mtree]]"
+
+    # Create startup-scripts/items
+    portstartupitem::startupitem_create
 }
 
 proc portdestroot::destroot_main {args} {
-    command_exec destroot
+    command_exec -callback portprogress::target_progress_callback destroot
     return 0
 }
 
 proc portdestroot::destroot_finish {args} {
-    global UI_PREFIX destroot prefix subport startupitem.create destroot.violate_mtree \
-           applications_dir frameworks_dir destroot.keepdirs destroot.delete_la_files \
-           os.platform os.version
+    global UI_PREFIX destroot prefix subport destroot.violate_mtree \
+           applications_dir frameworks_dir destroot.keepdirs destroot.delete_la_files
     variable oldmask
 
-    # Create startup-scripts/items
-    if {[tbool startupitem.create]} {
-        package require portstartupitem 1.0
-        portstartupitem::startupitem_create
-    }
-
     foreach fileToDelete {share/info/dir lib/charset.alias} {
-        if [file exists "${destroot}${prefix}/${fileToDelete}"] {
+        if {[file exists "${destroot}${prefix}/${fileToDelete}"]} {
             ui_debug "Deleting stray ${fileToDelete} file."
             file delete "${destroot}${prefix}/${fileToDelete}"
         }
@@ -170,28 +144,40 @@ proc portdestroot::destroot_finish {args} {
 
     # Prevent overlinking due to glibtool .la files: https://trac.macports.org/ticket/38010
     ui_debug "Fixing glibtool .la files in destroot for ${subport}"
-    fs-traverse -depth fullpath ${destroot} {
-        if {[file extension $fullpath] == ".la" && [file type $fullpath] == "file"} {
+    set la_file_list [list]
+    fs-traverse -depth fullpath [list $destroot] {
+        if {[file extension $fullpath] eq ".la" && ([file type $fullpath] eq "file" || [file type $fullpath] eq "link")} {
+            if {[file type $fullpath] eq "link" && [file pathtype [file link $fullpath]] ne "relative"} {
+                # prepend $destroot to target of absolute symlinks
+                set checkpath ${destroot}[file link $fullpath]
+            } else {
+                set checkpath $fullpath
+            }
             # Make sure it is from glibtool ... "a libtool library file" will appear in the first line
-            if {![catch {set fp [open $fullpath]}]} {
+            if {![catch {set fp [open $checkpath]}]} {
                 if {[gets $fp line] > 0 && [string first "a libtool library file" $line] != -1} {
-                    if {${destroot.delete_la_files}} {
-                        ui_debug "Removing [file tail $fullpath]"
-                        file delete -force ${fullpath}
-                    } else {
-                        ui_debug "Clearing dependency_libs in [file tail $fullpath]"
-                        reinplace "/dependency_libs/ s/'.*'/''/" ${fullpath}
-                    }
+                    lappend la_file_list $fullpath
                 }
             } else {
-                ui_debug "Failed to open $fullpath"
+                ui_debug "Failed to open $checkpath"
             }
             catch {close $fp}
         }
     }
+    foreach fullpath $la_file_list {
+        if {${destroot.delete_la_files}} {
+            ui_debug "Removing [file tail $fullpath]"
+            file delete -force ${fullpath}
+        } elseif {[file type $fullpath] eq "file"} {
+            ui_debug "Clearing dependency_libs in [file tail $fullpath]"
+            reinplace -q "/dependency_libs/ s/'.*'/''/" ${fullpath}
+        }
+    }
 
     # Prune empty directories in ${destroot}
     foreach path ${destroot.keepdirs} {
+        # Prepend $destroot if $path isn't already inside it
+        set path [::fileutil::jail $destroot $path]
         if {![file isdirectory ${path}]} {
             xinstall -m 0755 -d ${path}
         }
@@ -199,8 +185,8 @@ proc portdestroot::destroot_finish {args} {
             xinstall -c -m 0644 /dev/null ${path}/.turd_${subport}
         }
     }
-    fs-traverse -depth dir ${destroot} {
-        if {[file type $dir] == "directory"} {
+    fs-traverse -depth dir [list ${destroot}] {
+        if {[file type $dir] eq "directory"} {
             catch {file delete $dir}
         }
     }
@@ -216,36 +202,43 @@ proc portdestroot::destroot_finish {args} {
     }
 
     # Compress all manpages with gzip (instead)
-    set manpath "${destroot}${prefix}/share/man"
-    set gzip [findBinary gzip ${portutil::autoconf::gzip_path}]
-    set gunzip "$gzip -d"
-    set bunzip2 "[findBinary bzip2 ${portutil::autoconf::bzip2_path}] -d"
-    if {[file isdirectory ${manpath}] && [file type ${manpath}] == "directory"} {
+    set manpath ${destroot}${prefix}/share/man
+    if {[file isdirectory ${manpath}] && [file type ${manpath}] eq "directory"} {
         ui_info "$UI_PREFIX [format [msgcat::mc "Compressing man pages for %s"] ${subport}]"
+
+        set gzip [findBinary gzip ${portutil::autoconf::gzip_path}]
+        set gunzip "$gzip -d"
+        set bunzip2 "[findBinary bzip2 ${portutil::autoconf::bzip2_path}] -d"
+
         set found 0
         set manlinks [list]
+        set mandir_re {^(cat|man)(.)$}
+
         foreach mandir [readdir "${manpath}"] {
-            if {![regexp {^(cat|man)(.)$} ${mandir} match ignore manindex]} { continue }
+            if {![regexp ${mandir_re} ${mandir} match ignore manindex]} { continue }
+            set gzfile_re "^(.*\[.\]${manindex}\[a-z\]*)\[.\]gz\$"
+            set bz2file_re "^(.*\[.\]${manindex}\[a-z\]*)\[.\]bz2\$"
+            set normalfile_re "\[.\]${manindex}\[a-z\]*\$"
             set mandirpath [file join ${manpath} ${mandir}]
-            if {[file isdirectory ${mandirpath}] && [file type ${mandirpath}] == "directory"} {
+            if {[file isdirectory ${mandirpath}] && [file type ${mandirpath}] eq "directory"} {
                 ui_debug "Scanning ${mandir}"
                 foreach manfile [readdir ${mandirpath}] {
                     set manfilepath [file join ${mandirpath} ${manfile}]
-                    if {[file isfile ${manfilepath}] && [file type ${manfilepath}] == "file"} {
-                        if {[regexp "^(.*\[.\]${manindex}\[a-z\]*)\[.\]gz\$" ${manfile} gzfile manfile]} {
+                    if {[file isfile ${manfilepath}] && [file type ${manfilepath}] eq "file"} {
+                        if {[regexp ${gzfile_re} ${manfile} gzfile manfile]} {
                             set found 1
-                            system "cd ${manpath} && \
-                            $gunzip -f [file join ${mandir} ${gzfile}] && \
-                            $gzip -9vf [file join ${mandir} ${manfile}]"
-                        } elseif {[regexp "^(.*\[.\]${manindex}\[a-z\]*)\[.\]bz2\$" ${manfile} bz2file manfile]} {
+                            system -W ${manpath} \
+                            "$gunzip -f [shellescape [file join ${mandir} ${gzfile}]] && \
+                            $gzip -9vnf [shellescape [file join ${mandir} ${manfile}]]"
+                        } elseif {[regexp ${bz2file_re} ${manfile} bz2file manfile]} {
                             set found 1
-                            system "cd ${manpath} && \
-                            $bunzip2 -f [file join ${mandir} ${bz2file}] && \
-                            $gzip -9vf [file join ${mandir} ${manfile}]"
-                        } elseif {[regexp "\[.\]${manindex}\[a-z\]*\$" ${manfile}]} {
+                            system -W ${manpath} \
+                            "$bunzip2 -f [shellescape [file join ${mandir} ${bz2file}]] && \
+                            $gzip -9vnf [shellescape [file join ${mandir} ${manfile}]]"
+                        } elseif {[regexp ${normalfile_re} ${manfile}]} {
                             set found 1
-                            system "cd ${manpath} && \
-                            $gzip -9vf [file join ${mandir} ${manfile}]"
+                            system -W ${manpath} \
+                            "$gzip -9vnf [shellescape [file join ${mandir} ${manfile}]]"
                         }
                         set gzmanfile ${manfile}.gz
                         set gzmanfilepath [file join ${mandirpath} ${gzmanfile}]
@@ -257,7 +250,7 @@ proc portdestroot::destroot_finish {args} {
                                 file attributes ${gzmanfilepath} -permissions $desired
                             }
                         }
-                    } elseif {[file type ${manfilepath}] == "link"} {
+                    } elseif {[file type ${manfilepath}] eq "link"} {
                         lappend manlinks [file join ${mandir} ${manfile}]
                     }
                 }
@@ -265,11 +258,12 @@ proc portdestroot::destroot_finish {args} {
         }
         if {$found == 1} {
             # check man page links and rename/repoint them if necessary
+            set gzext_re "\[.\]gz\$"
             foreach manlink $manlinks {
                 set manlinkpath [file join $manpath $manlink]
                 # if link destination is not gzipped, check it
                 set manlinksrc [file readlink $manlinkpath]
-                if {![regexp "\[.\]gz\$" ${manlinksrc}]} {
+                if {![regexp ${gzext_re} ${manlinksrc}]} {
                     set mandir [file dirname $manlink]
                     set mandirpath [file join $manpath $mandir]
                     set pwd [pwd]
@@ -285,7 +279,7 @@ proc portdestroot::destroot_finish {args} {
                     # if gzipped destination exists, fix link
                     if {[file isfile ${mls_check}.gz]} {
                         # if actual link name does not end with gz, rename it
-                        if {![regexp "\[.\]gz\$" ${manlink}]} {
+                        if {![regexp ${gzext_re} ${manlink}]} {
                             ui_debug "renaming link: $manlink to ${manlink}.gz"
                             file rename $manlinkpath ${manlinkpath}.gz
                             set manlink ${manlink}.gz
@@ -305,7 +299,7 @@ proc portdestroot::destroot_finish {args} {
     }
 
     # test for violations of mtree
-    if { ${destroot.violate_mtree} != "yes" } {
+    if { ${destroot.violate_mtree} ne "yes" } {
         ui_debug "checking for mtree violations"
         set mtree_violation "no"
 
@@ -330,19 +324,19 @@ proc portdestroot::destroot_finish {args} {
                         if {$pfile eq "." || $pfile eq ".."} {
                             continue
                         }
-                        if {[lsearch -exact $prefixPaths $pfile] == -1} {
+                        if {$pfile ni $prefixPaths} {
                             ui_warn "violation by [file join $dfile $pfile]"
                             set mtree_violation "yes"
                         }
                     }
-                } elseif {[string equal -length [expr [string length $dfile] + 1] $dfile/ $prefix]} {
+                } elseif {[string equal -length [expr {[string length $dfile] + 1}] $dfile/ $prefix]} {
                     # we've found a subpath of our prefix
                     lpush pathsToCheck $dfile
                 } else {
                     set dir_allowed no
                     # these files are (at least potentially) outside of the prefix
-                    foreach dir "$applications_dir $frameworks_dir /Library/LaunchAgents /Library/LaunchDaemons /Library/StartupItems" {
-                        if {[string equal -length [expr [string length $dfile] + 1] $dfile/ $dir]} {
+                    foreach dir [list $applications_dir $frameworks_dir /Library/LaunchAgents /Library/LaunchDaemons /Library/StartupItems] {
+                        if {[string equal -length [expr {[string length $dfile] + 1}] $dfile/ $dir]} {
                             # it's a prefix of one of the allowed paths
                             set dir_allowed yes
                             break
@@ -368,13 +362,34 @@ proc portdestroot::destroot_finish {args} {
         }
 
         # abort here only so all violations can be observed
-        if { ${mtree_violation} != "no" } {
-            ui_warn "[format [msgcat::mc "%s violates the layout of the ports-filesystems!"] [option subport]]"
+        if { ${mtree_violation} ne "no" } {
+            ui_warn "[format [msgcat::mc "%s violates the layout of the ports-filesystems!"] $subport]"
             ui_warn "Please fix or indicate this misbehavior (if it is intended), it will be an error in future releases!"
             # error "mtree violation!"
         }
     } else {
-        ui_warn "[format [msgcat::mc "%s installs files outside the common directory structure."] [option subport]]"
+        ui_warn "[format [msgcat::mc "%s installs files outside the common directory structure."] $subport]"
+    }
+
+    # Work around apparent filesystem bug.
+    # https://trac.macports.org/ticket/67336
+    if {![catch {fs_clone_capable $destroot} result] && $result} {
+        global workpath
+        ui_debug "Applying sparse file lseek bug workaround"
+        try {
+            fs-traverse -depth fullpath [list $destroot] {
+                if {[file type $fullpath] eq "file" && [fileIsSparse $fullpath]} {
+                    ui_debug "Cloning $fullpath for workaround"
+                    clonefile $fullpath ${workpath}/.macports-sparse-workaround
+                    file delete ${workpath}/.macports-sparse-workaround
+                    if {![fileIsSparse $fullpath]} {
+                        ui_debug "$fullpath is no longer sparse"
+                    }
+                }
+            }
+        } on error {eMessage} {
+            ui_debug "Error while applying sparse file workaround: $eMessage"
+        }
     }
 
     # Restore umask
